/*
 * Copyright (c) 2006-2009 Bjorn Andersson <flex@kryo.se>, Erik Ekman <yarrick@kryo.se>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <check.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>

#include "test.h"

int
main()
{
	SRunner *runner;
	Suite *iodine;
	TCase *test;
	int failed;

	iodine = suite_create("iodine");

	test = test_base32_create_tests();
	suite_add_tcase(iodine, test);

	test = test_base64_create_tests();
	suite_add_tcase(iodine, test);

	test = test_dns_create_tests();
	suite_add_tcase(iodine, test);

	test = test_encoding_create_tests();
	suite_add_tcase(iodine, test);

 	test = test_read_create_tests();
	suite_add_tcase(iodine, test);

 	test = test_login_create_tests();
	suite_add_tcase(iodine, test);

 	test = test_user_create_tests();
	suite_add_tcase(iodine, test);

 	test = test_fw_query_create_tests();
	suite_add_tcase(iodine, test);

	runner = srunner_create(iodine);
	srunner_run_all(runner, CK_NORMAL);
	failed = srunner_ntests_failed(runner);

	srunner_free(runner);

	return (failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
}
